/*******************************************************************\

Copyright (c) 2008 Leiden University (LERC group at LIACS).
All rights reserved.

The use and distribution terms for this software are covered by the 
Common Public License 1.0 (http://opensource.org/licenses/cpl1.0.txt)
which can be found in the file LICENSE at the root of this distribution.
By using this software in any fashion, you are agreeing to be bound by 
the terms of this license.

You must not remove this notice, or any other, from this software.

\*******************************************************************/

#include "DCT.h"

/*
PreshiftDctMatrix() subtracts 128 (2048) from all 64 elements of an 8x8
matrix.  This results in a balanced DCT without any DC bias.
*/
void PreshiftDctMatrix(int *matrix, int shift)
{
        int *mptr;

	for(mptr=matrix;mptr<matrix+BLOCKSIZE;mptr++)
	{*mptr -= shift;}
}

/*
BoundDctMatrix() clips the Dct matrix such that it is no larger than
a 10 (1023) bit word or 14 bit word (4095).
*/
void BoundDctMatrix(int *matrix, int Bound)
{
        int *mptr;

	for(mptr=matrix;mptr<matrix+BLOCKSIZE;mptr++) {
		if (*mptr+Bound < 0)
			*mptr = -Bound;
		else if (*mptr-Bound > 0)
			*mptr = Bound;
	}
}


/**************************/
/* Intiger Arithmetic DCT */
/**************************/
void IntArithDct (int *block, int *outdata)
{
  int i,j,k;
  long s;
  long tmp[64];

  for (i=0; i<8; i++) {
     for (j=0; j<8; j++) {
        s = 0;
        for (k=0; k<8; k++) {
           s += (block[8*i+k])*(c[j][k]>>16);
        }
        tmp[8*i+j] = s >> 8;
	 }
  }


  for (i=0; i<8; i++) {
     for (j=0; j<8; j++) {
        s = 0;
        for (k=0; k<8; k++) {
          s += (c[i][k]>>16) * tmp[8*k+j];
        }                                                                                                                                                                                                                                                        ;
          outdata[8*i+j] = s >> 8;
     }
  }

  return;
}


void mainDCT(const TBlocks *input, TBlocks *output)
{

	int	DCTBound,DCTShift;
	DCTBound = ((DataPrecision)?16383:1024);
	DCTShift = ((DataPrecision)?2048:128);

	//Y1
	PreshiftDctMatrix((int*)(*input).Y1.pixel, DCTShift);     // Shift
	IntArithDct((int*)(*input).Y1.pixel, (*output).Y1.pixel);      // DCT
	BoundDctMatrix((*output).Y1.pixel, DCTBound);       // Bound, limit

	//Y2
	PreshiftDctMatrix((int*)(*input).Y2.pixel, DCTShift);     // Shift
	IntArithDct((int*)(*input).Y2.pixel, (*output).Y2.pixel);      // DCT
	BoundDctMatrix((*output).Y2.pixel, DCTBound);       // Bound, limit

	//U1
	IntArithDct((int*)(*input).U1.pixel, (*output).U1.pixel);      // DCT
	BoundDctMatrix((*output).U1.pixel, DCTBound);       // Bound, limit

	// V1
	IntArithDct((int*)(*input).V1.pixel, (*output).V1.pixel);      // DCT
	BoundDctMatrix((*output).V1.pixel, DCTBound);       // Bound, limit

	return;
}

