/*******************************************************************\

Copyright (c) 2008 Leiden University (LERC group at LIACS).
All rights reserved.

The use and distribution terms for this software are covered by the 
Common Public License 1.0 (http://opensource.org/licenses/cpl1.0.txt)
which can be found in the file LICENSE at the root of this distribution.
By using this software in any fashion, you are agreeing to be bound by 
the terms of this license.

You must not remove this notice, or any other, from this software.

\*******************************************************************/

#ifndef P_11_H
#define P_11_H

#include "math.h"
#include "yapi.h"

#include "aux_func.h"
#include <iostream>


class P_11 : public Process {
  private:

    // Input Gates and controllers
    InPort<tCH_10> IG_1;
    InPort<tCH_11> IG_2;
    InPort<tCH_12> IG_3;
    InPort<tCH_13> IG_4;

    // Output Gates
    OutPort<tCH_14> OG_1;

    // Parameters

    // Input Arguments 
    tCH_13 in_0ND_10;

    // Output Arguments 
    tCH_14 out_1ND_10;

  public:

    // Constructor 
    P_11(Id n,
    In<tCH_10>& IG_1_instance,
    In<tCH_11>& IG_2_instance,
    In<tCH_12>& IG_3_instance,
    In<tCH_13>& IG_4_instance,
    Out<tCH_14>& OG_1_instance) :
    Process(n),
    IG_1(id("IG_1"), IG_1_instance ),
    IG_2(id("IG_2"), IG_2_instance ),
    IG_3(id("IG_3"), IG_3_instance ),
    IG_4(id("IG_4"), IG_4_instance ),
    OG_1(id("OG_1"), OG_1_instance )
    {
    }; 

    const char* type() const {return "P_11";};

    void main() {

      for( int c0 =  ceil1(0); c0 <=  floor1(31 ); c0 += 1 ) {
        for( int c1 =  ceil1(1); c1 <=  floor1(16 ); c1 += 1 ) {
          for( int c2 =  ceil1(1); c2 <=  floor1(8 ); c2 += 1 ) {
              if( -c2 + 2 >= 0 ) {
                read(IG_1, in_0ND_10);

              }
              if( c2-3 >= 0 ) {
                if( -c2 + 4 >= 0 ) {
                  read(IG_2, in_0ND_10);

                }
              }
              if( c2-5 >= 0 ) {
                if( -c2 + 6 >= 0 ) {
                  read(IG_3, in_0ND_10);

                }
              }
              if( c2-7 >= 0 ) {
                read(IG_4, in_0ND_10);

              }

              _mainVLE(in_0ND_10, out_1ND_10) ;
              execute("_mainVLE");


              write( OG_1, out_1ND_10);

          } // for c2
        } // for c1
      } // for c0

    } // main
};

#endif
