/*******************************************************************\

Copyright (c) 2008 Leiden University (LERC group at LIACS).
All rights reserved.

The use and distribution terms for this software are covered by the 
Common Public License 1.0 (http://opensource.org/licenses/cpl1.0.txt)
which can be found in the file LICENSE at the root of this distribution.
By using this software in any fashion, you are agreeing to be bound by 
the terms of this license.

You must not remove this notice, or any other, from this software.

\*******************************************************************/

#include "Q.h"

// Quantize() quantizes an input matrix and puts the output in matrix.
void Quantize(int *matrix, int* qmatrix)
{
	int *mptr;
	for(mptr=matrix;mptr<matrix+BLOCKSIZE;mptr++) {
		if (*mptr > 0) {
			// Rounding is different for +/- coeffs
			*mptr = (*mptr + *qmatrix/2)/ (*qmatrix);
			qmatrix++;
		} else {
			*mptr = (*mptr - *qmatrix/2)/ (*qmatrix);
			qmatrix++;
		}
	}
}

/*
ZigzagMatrix() performs a zig-zag translation on the input imatrix
and puts the output in omatrix.
*/
void ZigzagMatrix(int *imatrix, int *matrix)
{
	int omatrix[BLOCKSIZE];
        int i;
	int *tptr;

	for(tptr=zigzag_index;tptr<zigzag_index+BLOCKSIZE;tptr++) {
		omatrix[*tptr] = *(imatrix++);
	}

	for(i = 0; i < BLOCKSIZE; i++) {
		matrix[i] = omatrix[i];
	}
}

void mainQ(const TBlocks  *input, TBlocks *output)
{

	// Y1
	Quantize((int*)(*input).Y1.pixel, LuminanceQTable.QCoef);
	ZigzagMatrix((int*)(*input).Y1.pixel, (*output).Y1.pixel);

	// Y2
	Quantize((int*)(*input).Y2.pixel, LuminanceQTable.QCoef);
	ZigzagMatrix((int*)(*input).Y2.pixel, (*output).Y2.pixel);

	// U1
	Quantize((int*)(*input).U1.pixel, ChrominanceQTable.QCoef);
	ZigzagMatrix((int*)(*input).U1.pixel, (*output).U1.pixel);

	// V1
	Quantize((int*)(*input).V1.pixel, ChrominanceQTable.QCoef);
	ZigzagMatrix((int*)(*input).V1.pixel, (*output).V1.pixel);

}


